--[[
	Project: BookOfTheFallen
	Version: 1.0.0
	Author: NerfedWar (http://www.nerfedwar.net - nerfed.war@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]

-- initialise globals
if not BOTF then BOTF = {} end
if not BOTF.Book then BOTF.Book = {} end

-- @field define the PVE class
BOTF.Book.PVE = class(BOTF.Book.BasicPage)

-- PVE class constructor
-- @param book the Book pagesFrame
function BOTF.Book.PVE:init(book)

  -- @field store a reference to the Book pagesFrame
  self.theBook = book

  self.currentListPage = 1
  self.recordsPerPage = 40
  self.listPageSortField = "name" -- TODO allow sorting on other columns

  -- create list header
  self.listPageTextHeader1 = UI.CreateFrame("Text", "BOTFUIselflistPageTextHeader1", self.theBook.bookFrame)
  self.listPageTextHeader1:SetWidth(400)
  self.listPageTextHeader1:SetHeight(23)
  self.listPageTextHeader1:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 40, 15)
  self.listPageTextHeader1:SetLayer(2)
  self.listPageTextHeader1:SetText("  Name		             Calling   Level  Score")
  self.listPageTextHeader1:SetFont("BOTF", "MONOFONT.TTF")
  self.listPageTextHeader1:SetFontSize(18)
  self.listPageTextHeader1:SetFontColor(0.4, 0.3, 0.3, 0.8)
  self.listPageTextHeader1:SetVisible(false)
  self.listPageTextHeader2 = UI.CreateFrame("Text", "BOTFUIselflistPageTextHeader2", self.theBook.bookFrame)
  self.listPageTextHeader2:SetWidth(400)
  self.listPageTextHeader2:SetHeight(23)
  self.listPageTextHeader2:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 450, 15)
  self.listPageTextHeader2:SetLayer(2)
  self.listPageTextHeader2:SetText("  Name		             Calling   Level  Score")
  self.listPageTextHeader2:SetFont("BOTF", "MONOFONT.TTF")
  self.listPageTextHeader2:SetFontSize(18)
  self.listPageTextHeader2:SetFontColor(0.4, 0.3, 0.3, 0.8)
  self.listPageTextHeader2:SetVisible(false)

  -- create listing frames
  self.listPageTextFrames = {}
  for i = 1, self.recordsPerPage, 1 do
    local frame = UI.CreateFrame("Text", "BOTFUIListPageTextFrame" .. i, self.theBook.bookFrame)
    frame:SetWidth(400)
    frame:SetHeight(23)
    local y = 0
    local x = 0
    if i <= self.recordsPerPage/2 then
      x = 40
      y = 40 + (i * 15)
    else
      x = 456
      y = 40 + ((i - self.recordsPerPage/2) * 15)
    end
    frame:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", x, y)
    frame:SetLayer(2)
    frame:SetText("")
    frame:SetFont("BOTF", "MONOFONT.TTF")
    frame:SetFontSize(18)
    frame:SetFontColor(0.4, 0.3, 0.3, 0.8)
    frame:SetVisible(false)
    frame:EventAttach(Event.UI.Input.Mouse.Left.Click,function(frame,h)
      -- event handler for left click
    end,"Frame LeftClick")

    self.listPageTextFrames[i] = frame
  end

  -- create list next/prev arrows
  self.list_prev = UI.CreateFrame("Texture", "BOTFUIListPrev", self.theBook.bookFrame)
  self.list_prev:SetTexture("BOTF", "media/prev_button_1.png")
  --self.list_prev:ResizeToTexture()
  self.list_prev:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 143, 450)
  self.list_prev:SetLayer(2)
  self.list_prev:SetVisible(false)
  self.list_prev.object = self
  self.list_prev:EventAttach(Event.UI.Input.Mouse.Cursor.In,function(self,h)
    self:SetTexture("BOTF", "media/prev_button_2.png")
  end, "Previous Mouse In")
  self.list_prev:EventAttach(Event.UI.Input.Mouse.Cursor.Out,function(self,h)
    self:SetTexture("BOTF", "media/prev_button_1.png")
  end, "Prevous Mouse Out")

  self.list_next = UI.CreateFrame("Texture", "BOTFUIListNext", self.theBook.bookFrame)
  self.list_next:SetTexture("BOTF", "media/next_button_1.png")
  --self.list_next:ResizeToTexture()
  self.list_next:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 747, 450)
  self.list_next:SetLayer(2)
  self.list_next:SetVisible(false)
  self.list_next.object = self
  self.list_next:EventAttach(Event.UI.Input.Mouse.Cursor.In,function(self,h)
    self:SetTexture("BOTF", "media/next_button_2.png")
  end, "Next Mouse In")

  self.list_next:EventAttach(Event.UI.Input.Mouse.Cursor.Out,function(self,h)
    self:SetTexture("BOTF", "media/next_button_1.png")
  end, "Next Mouse Out")

  -- create page numbers
  self.odd_pnum = UI.CreateFrame("Text", "BOTFUIListPageOddNum", self.theBook.bookFrame)
  self.odd_pnum:SetWidth(100)
  self.odd_pnum:SetHeight(23)
  self.odd_pnum:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 253, 455)
  self.odd_pnum:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 253, 455)
  self.odd_pnum:SetLayer(2)
  self.odd_pnum:SetText("Page x-1")
  self.odd_pnum:SetFontSize(14)
  self.odd_pnum:SetFontColor(0.4, 0.3, 0.3, 0.8)
  self.odd_pnum:SetVisible(false)
  self.even_pnum = UI.CreateFrame("Text", "BOTFUIListPageEvenNum", self.theBook.bookFrame)
  self.even_pnum:SetWidth(100)
  self.even_pnum:SetHeight(23)
  self.even_pnum:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 598, 455)
  self.even_pnum:SetLayer(2)
  self.even_pnum:SetText("Page x")
  self.even_pnum:SetFontSize(14)
  self.even_pnum:SetFontColor(0.4, 0.3, 0.3, 0.8)
  self.even_pnum:SetVisible(false)
  
  self.list_prev:EventAttach(Event.UI.Input.Mouse.Left.Click,function(self,h)
      self.object:show(self.object.currentListPage - 1)
  end,"Previous LeftClick")
    
  self.list_next:EventAttach(Event.UI.Input.Mouse.Left.Click,function(self,h)
      self.object:show(self.object.currentListPage + 1)
  end, "Next LeftClick")
  
end

-- update and display the UI elements
function BOTF.Book.PVE:show(page)

  -- set the background of the pageFrame
  self.theBook.pagesFrame:SetTexture("BOTF", "media/book_list_pve.png")

  if page then self.currentListPage=page end

  self.listPageTextHeader1:SetVisible(true)
  self.listPageTextHeader2:SetVisible(true)
  for i = 1, self.recordsPerPage, 1 do self.listPageTextFrames[i]:SetVisible(true) end
  self.odd_pnum:SetVisible(true)
  self.even_pnum:SetVisible(true)

  local pve_db = {}
  for i, v in orderedPairs(BOTF_DB) do
    if string.sub(i, 1, 2) ~= "__" and i~="" then
      if v.player==false or not v.player then
        pve_db[i] = BOTF_DB[i]
      end
    end
  end

  -- clear all text fields
  for i = 1, self.recordsPerPage, 1 do
    self.listPageTextFrames[i]:SetText("")
  end

  -- loop through table
  local count = 1
  local totalCount = 1
  local playerLevel = Inspect.Unit.Detail('player').level
  for i, v in orderedPairs(pve_db) do

    if string.sub(i, 1, 2) ~= "__" and i~="" then

      -- only output our page
      if totalCount > self.recordsPerPage * (self.currentListPage - 1) and totalCount <= self.recordsPerPage * self.currentListPage then

        local score = 0
        if v.level~='unknown' and tonumber(v.level)>= tonumber(playerLevel) then
          score = (((1 * v.statistics.Victory_SoloKills) + v.statistics.Victory_KillingBlows + v.statistics.Victory_DamageAssists + v.statistics.Victory_DistractAssists + v.statistics.Victory_HealAssists) - ((1 * v.statistics.Defeat_SoloKills) + v.statistics.Defeat_KillingBlows + v.statistics.Defeat_DamageAssists + v.statistics.Defeat_DistractAssists + v.statistics.Defeat_HealAssists))
        end
        local frame = self.listPageTextFrames[count]
        local shortName = i

        if string.len(i) > 20 then
          shortName = string.sub(i, 1, 20)
          shortName = string.rpad(shortName, 23, ".")
        end
        local isSession = "  "
        if BOTF_DB[i].session then
          isSession = "* "
        end

        if not v.calling then
          v.calling="critter"
        end

        if v.level=='unknown' then
          v.level='0'
        end

        if v.level=='??' then
            v.level='0'
        end

        frame:SetText(isSession .. string.rpad(shortName, 28, " ") .. string.rpad(v.calling, 11, " ") .. string.rpad(v.level, 4, " ") .. string.lpad(score, 5, " "))
        frame.object = self
        frame:EventAttach(Event.UI.Input.Mouse.Left.Click,function(self,h)
          self.object.theBook:setPage("details", i)
        end, "List Left Click")

        frame:EventAttach(Event.UI.Input.Mouse.Cursor.In,function(frame,h)
          frame:SetFontColor(0.8, 0.3, 0.3, 0.8)
        end, "List Mouse In")

        frame:EventAttach(Event.UI.Input.Mouse.Cursor.Out,function(frame,h)
          frame:SetFontColor(0.4, 0.3, 0.3, 0.8)
        end, "List Mouse Out")

        count = count + 1
      end

      totalCount = totalCount + 1
    end
  end

  -- prev and next buttons
  if self.currentListPage ~= 1 then
    self.list_prev:SetVisible(true)
    
  else
    self.list_prev:SetVisible(false)
  end
  local totalPages = (totalCount - 1) / self.recordsPerPage
  if totalPages > self.currentListPage then
    self.list_next:SetVisible(true)
    
  else
    self.list_next:SetVisible(false)
  end

  -- page numbers
  self.odd_pnum:SetText("Page " .. (self.currentListPage * 2) - 1)
  self.even_pnum:SetText("Page " .. (self.currentListPage * 2))
end

-- hide the UI elements
function BOTF.Book.PVE:hide()

  -- reset the contents of the text frames and hide them
  self.listPageTextHeader1:SetVisible(false)
  self.listPageTextHeader2:SetVisible(false)
  self.odd_pnum:SetVisible(false)
  self.even_pnum:SetVisible(false)
  for i = 1, self.recordsPerPage, 1 do
    self.listPageTextFrames[i]:SetVisible(false)
  end
  self.list_prev:SetVisible(false)
  self.list_next:SetVisible(false)

end

