--[[
	Project: BookOfTheFallen
	Version: 1.0.0
	Author: NerfedWar (http://www.nerfedwar.net - nerfed.war@gmail.com)
	
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
]]

------------------------
--[[ PUBLIC MEMBERS ]] --
------------------------
BOTF = {}
BOTF_DB = {}
BOTF_CONFIG = {}

-------------------------
--[[ PRIVATE MEMBERS ]] --
-------------------------

local theBook

local theAlert

-- @field table of unit detail information for players you have recently interacted with.
local unitDetailsCache = {}

-- @field holds who has damaged who, when and by how much
local recentDamage = {}

-- @field holds who has healed who, when and by how much
local recentHeals = {}

-- @field assist window in seconds
local assistWindow = 10

playerDetail = nil

------------------------
--[[ PUBLIC METHODS ]] --
------------------------
function BOTF.show()
  -- set the page to home and show
  theBook:setPage('home')
  theBook:show()
end

function BOTF.hide() theBook:hide() end

function BOTF.toggle()
  if theBook.bookFrame:GetVisible() == false then
    BOTF.show()
  else
    BOTF.hide()
  end
end

function BOTF.reset()
  BOTF_DB = {}
  BOTF_CONFIG = {}
end

-------------------------
--[[ PRIVATE METHODS ]] --
-------------------------

-- Addon initialisation
-- 1. Registers event handlers to deal with damage, heal and death events.
-- 2. Removes all session information fromt the database.
-- 3. Outputs initialisation message.
local function init()

  -- create the alert
  theAlert = BOTF.Alert()

  -- create book
  theBook = BOTF.Book()

  Command.Event.Attach(Event.System.Update.Begin, theAlert.Update, "AlertUpdate" )

  -- @event register event to deal with assists and distracts
  Command.Event.Attach(Event.Combat.Damage, function(hEvent,botf_damage) BOTF.doDamage(botf_damage) end, "BOTF.doDamage" )

  -- @event register handler to deal with deaths
  Command.Event.Attach(Event.Combat.Death, function(hEvent,botf_death) BOTF.doDeath(botf_death) end, "BOTF.doDeath" )

  -- @event register handler to deal with heals
  Command.Event.Attach(Event.Combat.Heal, function(hEvent,botf_heal) BOTF.doHeal(botf_heal) end, "BOTF.doHeal" )

  -- @event register handler to deal with new units being seen
  Command.Event.Attach(Event.Unit.Remove, function(hEvent,botf_remove) BOTF.doNewUnit(botf_remove) end, "BOTF.doNewUnit" )

  -- loop through DB and set all session vars to false
  for i, v in pairs(BOTF_DB) do
    BOTF_DB[i].session = false
    BOTF_DB[i]['session_statistics'] = {}
    BOTF_DB[i]['session_statistics']['Victory_SoloKills'] = 0
    BOTF_DB[i]['session_statistics']['Victory_KillingBlows'] = 0
    BOTF_DB[i]['session_statistics']['Victory_DamageAssists'] = 0
    BOTF_DB[i]['session_statistics']['Victory_DistractAssists'] = 0
    BOTF_DB[i]['session_statistics']['Victory_HealAssists'] = 0
    BOTF_DB[i]['session_statistics']['Defeat_SoloKills'] = 0
    BOTF_DB[i]['session_statistics']['Defeat_KillingBlows'] = 0
    BOTF_DB[i]['session_statistics']['Defeat_DamageAssists'] = 0
    BOTF_DB[i]['session_statistics']['Defeat_DistractAssists'] = 0
    BOTF_DB[i]['session_statistics']['Defeat_HealAssists'] = 0
  end

  -- Register the slash commands
  table.insert(Command.Slash.Register("botf"), { BOTF.show, "BOTF", "Slash command" })
  table.insert(Command.Slash.Register("botfreset"), { BOTF.reset, "BOTF", "Slash command" })

  print("Book of the Fallen loaded. Click the book icon or type /botf to launch.")
end

--
local isApiInitialised = false
local function isApiInitialisedLoop()
  if isApiInitialised then return end
  if Inspect.Unit.Detail('player') ~= nil then
    isApiInitialised = true
    init()
  end
end

-- the addon has loaded but the addon API is not fully initialised (annoying)
-- start a loop to check
local function addonLoaded()
  Command.Event.Attach(Event.System.Update.Begin,  isApiInitialisedLoop, "isApiInitialised" )
end

-- Called when there is a unit addevent
-- 1. Stores the unit detail information for the caster and target of the damage event for future use.
-- @param evt a table containing the following combat damage event information (as of Rift version 1.5): ability, abilityname, caster unit id, damage, target unit id.

function BOTF.doNewUnit(evt)
  for i, _ in pairs(evt) do
    if i ~= false then
      -- store unit information into cache if required
      if unitDetailsCache[i] == nil then
        if Inspect.Unit.Detail(i) ~= nil then
          unitDetailsCache[i] = Inspect.Unit.Detail(i)
        end
      end
    end
  end
end


-- Called when there is a combat damage event
-- 1. Stores the event information into the recentDamage table for future analysis as and when a death event occurs.
-- 2. Stores the unit detail information for the caster and target of the damage event for future use.
-- @param evt a table containing the following combat damage event information (as of Rift version 1.5): ability, abilityname, caster unit id, damage, target unit id.
function BOTF.doDamage(evt)

  -- get my unit id
  local myUnitId = Inspect.Unit.Lookup('player')

  -- get current timestamp
  local time = Inspect.Time.Real()

  -- store the event information
  recentDamage[time] = evt

  -- store unit information into cache if required
  if unitDetailsCache[evt.caster] == nil then
    if Inspect.Unit.Detail(evt.caster) ~= nil then
      unitDetailsCache[evt.caster] = Inspect.Unit.Detail(evt.caster)
    end
  end
  if unitDetailsCache[evt.target] == nil then
    if Inspect.Unit.Detail(evt.target) ~= nil then
      unitDetailsCache[evt.target] = Inspect.Unit.Detail(evt.target)
    end
  end

  -- clean out recentDamage table of old records
  recentDamage = BOTF.cleanOld(recentDamage)
end

-- Called when there is a combat heal event
-- 1. Stores the event information into the recentHeal table for future analysis as and when a death event occurs.
-- 2. Stores the unit detail information for the caster and target of the dhealge event for future use.
-- @param evt a table containing the following combat heal event information (as of Rift version 1.5): ability, abilityname, caster unit id, heal, target unit id.
function BOTF.doHeal(evt)

  -- get my unit id
  local myUnitId = Inspect.Unit.Lookup('player')

  -- get current timestamp
  local time = Inspect.Time.Real()

  -- store the event information
  recentHeals[time] = evt

  -- store unit information into cache if required
  if unitDetailsCache[evt.caster] == nil then
    if Inspect.Unit.Detail(evt.caster) then
      unitDetailsCache[evt.caster] = Inspect.Unit.Detail(evt.caster)
    end
  end
  if unitDetailsCache[evt.target] == nil then
    if Inspect.Unit.Detail(evt.target) then
      unitDetailsCache[evt.target] = Inspect.Unit.Detail(evt.target)
    end
  end

  -- clean out recentDamage table of old records
  recentHeals = BOTF.cleanOld(recentHeals)
end


-- Called when a death occurs
-- 1. Get the unit information for the caster and target (from the cache if needs be).
-- 2. Check we don't have missing information and that we did not kill ourselves.
-- 3. Calculate the damage,distract,heal percentage of the player against the target.
-- 4. Store data into database.
-- @param evt a table containing the death event information (as of Rift 1.5): caster unit id, target unit id.
function BOTF.doDeath(evt)

  -- get killer and deceased
  local killerId = evt.caster
  local deceasedId = evt.target

  -- get the player id
  local playerId = Inspect.Unit.Lookup('player')

  -- hack to deal with duels and missing killers.
  if not killerId and deceasedId==playerId then
    print("No killer Id, assuming player.target")
    killerId =  Inspect.Unit.Lookup('player.target')
  end

  -- get details and fallback to cache
  local killerDetail = false
  local deceasedDetail = false
  if killerId ~= nil then killerDetail = Inspect.Unit.Detail(killerId) end
  if deceasedId ~= nil then deceasedDetail = Inspect.Unit.Detail(deceasedId) end

  if not killerDetail then killerDetail = unitDetailsCache[killerId] end
  if not deceasedDetail then deceasedDetail = unitDetailsCache[deceasedId] end

  -- we can't record this event if we are missing information
  if not killerDetail then killerDetail = BOTF.getUnknown() end
  if not deceasedDetail then deceasedDetail = BOTF.getUnknown() end

  -- Victory! Work out player contribution against deceased.
  if deceasedId ~= playerId then

    -- allies got a kill

    -- if we are against a pve mob then ignore the death if it is not a named mob.
    if not deceasedDetail.player then
        if deceasedDetail.tier == null then
            return
        end
    end


    local damageByPlayerToDeceased = 0
    local damageByEveryoneToDeceased = 0
    local damageByDeceasedToPlayer = 0
    local damageByDeceasedToEverybody = 0

    local healsByPlayerToKiller = 0
    local healsByEveryoneToKiller = 0

    for i, v in pairs(recentDamage) do

      -- get damager and damaged
      local damager = v.caster
      local damaged = v.target

      -- calculate total raw damage
      local damage = 0
      if v.damage then damage = damage + v.damage end
      if v.damageAbsorbed then damage = damage + v.damageAbsorbed end
      if v.damageBlocked then damage = damage + v.damageBlocked end
      if v.damageDeflected then damage = damage + v.damageDeflected end
      if v.damageModified then damage = damage + v.damageModified end

      -- player does the damage to deceased
      if damager == playerId and damaged == deceasedId then
        damageByPlayerToDeceased = damageByPlayerToDeceased + damage
      elseif
        -- deceased does damage to the player
      damager == deceasedId and damaged == playerId then
        damageByDeceasedToPlayer = damageByDeceasedToPlayer + damage
      end

      -- if anyone does damage to the deceased
      if damaged == deceasedId then
        damageByEveryoneToDeceased = damageByEveryoneToDeceased + damage
      elseif
        -- if the deceased does damage to anyone
      damager == deceasedId then
        damageByDeceasedToEverybody = damageByDeceasedToEverybody + damage
      end
    end


    for i, v in pairs(recentHeals) do

      -- get healer and healed
      local healer = v.caster
      local healed = v.target

      -- calculate total raw healing
      local heal = 0
      if v.heal then heal = heal + v.heal end

      -- player does the heal to killer
      if healer == playerId and healed == killerId then
        healsByPlayerToKiller = healsByPlayerToKiller + heal
      end

      -- anyone does healing to the killer
      if healed == killerId then
        healsByEveryoneToKiller = healsByEveryoneToKiller + heal
      end
    end

    -- calculate contributions

    -- player damage ratio: player MUST do damage to the target
    local playerDamageRatio = 0
    if damageByPlayerToDeceased > 0 then
      playerDamageRatio = damageByPlayerToDeceased / damageByEveryoneToDeceased
    else
      playerDamageRatio = 0
    end

    -- target must do damage to the player
    local playerDistractRatio = 0
    if damageByDeceasedToPlayer > 0 then
      playerDistractRatio = damageByDeceasedToPlayer / damageByDeceasedToEverybody
    else
      playerDistractRatio = 0
    end

    -- player must heal killer
    local playerHealRatio = 0
    if healsByPlayerToKiller > 0 then
      playerHealRatio = healsByPlayerToKiller / healsByEveryoneToKiller
    else
      playerHealRatio = 0
    end
    local vKillingBlow = (killerId == playerId) and 1 or 0
    local vSolo = (vKillingBlow == 1 and (playerDamageRatio == 1 or damageByEveryoneToDeceased == 0) and (playerDistractRatio == 1 or damageByDeceasedToEverybody == 0) and (playerHealRatio == 1 or healsByEveryoneToKiller == 0)) and 1 or 0
    local vDamageAssist = playerDamageRatio > 0.5 and 1 or 0
    local vDistractAssist = playerDistractRatio > 0.5 and 1 or 0
    local vHealAssist = playerHealRatio > 0.5 and 1 or 0

    -- check for solo or assist/killingblow
    local iconFlags = {}
    if vSolo == 1 and not ptSolo then
      table.insert(iconFlags, "SoloKill")
      BOTF.updateRecord(deceasedDetail.name, deceasedDetail.calling, deceasedDetail.role, deceasedDetail.level, deceasedDetail.player, deceasedDetail.locationName, "Victory_SoloKills")
    end
    if vKillingBlow == 1 then
      table.insert(iconFlags, "KillingBlow")
      BOTF.updateRecord(deceasedDetail.name, deceasedDetail.calling, deceasedDetail.role, deceasedDetail.level, deceasedDetail.player, deceasedDetail.locationName, "Victory_KillingBlows")
    end
    if vDamageAssist == 1 then
      table.insert(iconFlags, "DamageAssist")
      BOTF.updateRecord(deceasedDetail.name, deceasedDetail.calling, deceasedDetail.role, deceasedDetail.level, deceasedDetail.player, deceasedDetail.locationName, "Victory_DamageAssists")
    end
    if vDistractAssist == 1 then
      table.insert(iconFlags, "DistractAssist")
      BOTF.updateRecord(deceasedDetail.name, deceasedDetail.calling, deceasedDetail.role, deceasedDetail.level, deceasedDetail.player, deceasedDetail.locationName, "Victory_DistractAssists")
    end
    if vHealAssist == 1 then
      table.insert(iconFlags, "HealAssist")
      BOTF.updateRecord(deceasedDetail.name, deceasedDetail.calling, deceasedDetail.role, deceasedDetail.level, deceasedDetail.player, deceasedDetail.locationName, "Victory_HealAssists")
    end

    if vSolo == 1 then
      theAlert:setMessage("Solo Kill", "You single-handedly crushed " .. deceasedDetail.name, iconFlags)
    elseif vKillingBlow == 1 then
      theAlert:setMessage("Killing Blow", "You got the killing blow on " .. deceasedDetail.name, iconFlags)
    elseif vDamageAssist == 1 or vDistractAssist == 1 or vHealAssist == 1 then
      theAlert:setMessage("Assisted Kill", "You assisted in the slaughter of " .. deceasedDetail.name, iconFlags)
    end

  else
    -- Enemies got a kill

    -- if we are against a pve mob then ignore the death if it is not a named mob.
    if not killerDetail.player then
        if killerDetail.tier == null then
          return
        end
    end

    local damageByEnemyToDeceasedPlayer = {}
    local damagebyAllEnemiesToDeceasedPlayer = 0
    local damageByDeceasedPlayerToEnemy = {}
    local damageByDeceasedPlayerToAllEnemies = 0

    local healingOfKillerByEnemy = {}
    local healingOfKillerByAllEnemies = 0

    for i, v in pairs(recentDamage) do

      -- get damager and damaged
      local damager = v.caster
      local damaged = v.target

      -- calculate total raw damage
      local damage = 0
      if v.damage then damage = damage + v.damage end
      if v.damageAbsorbed then damage = damage + v.damageAbsorbed end
      if v.damageBlocked then damage = damage + v.damageBlocked end
      if v.damageDeflected then damage = damage + v.damageDeflected end
      if v.damageModified then damage = damage + v.damageModified end

      if not damageByEnemyToDeceasedPlayer[damager] or damageByEnemyToDeceasedPlayer[damager] == nil then damageByEnemyToDeceasedPlayer[damager] = 0 end
      if not damageByDeceasedPlayerToEnemy[damaged] or damageByDeceasedPlayerToEnemy[damaged] == nil then damageByDeceasedPlayerToEnemy[damaged] = 0 end

      if damaged == playerId then
        -- damage done to player
        damageByEnemyToDeceasedPlayer[damager] = damageByEnemyToDeceasedPlayer[damager] + damage
        damagebyAllEnemiesToDeceasedPlayer = damagebyAllEnemiesToDeceasedPlayer + damage
      elseif damager == playerId then
        -- deceased does damage to the player
        damageByDeceasedPlayerToEnemy[damaged] = damageByDeceasedPlayerToEnemy[damaged] + damage
        damageByDeceasedPlayerToAllEnemies = damageByDeceasedPlayerToAllEnemies + damage
      end
    end

    if deceasedId == playerId then
      for i, v in pairs(recentHeals) do

        -- get healer and healed
        local healer = v.caster
        local healed = v.target

        if healer and healed then

          -- calculate total raw healing
          local heal = 0
          if v.heal then heal = heal + v.heal end

          -- anyone does healing to the killer
          if healed == killerId and deceasedId == playerId then
            if not healingOfKillerByEnemy[healer] then
              healingOfKillerByEnemy[healer] = 0
            end
            healingOfKillerByEnemy[healer] = healingOfKillerByEnemy[healer] + heal
            healingOfKillerByAllEnemies = healingOfKillerByAllEnemies + heal
          end
        end
      end
    end

    -- boolean variables to ensure we don't assign the same reward twice
    local pSolo = false
    local pKillingBlow = false
    local pDamageAssist = false
    local pDistractAssist = false
    local pHealAssist = false

    -- loop through all enemies that have healed the killer of the player
    local healerUnitId = false
    for i, _ in pairs(healingOfKillerByEnemy) do

      local enemyHealRatio = 0
      if healingOfKillerByEnemy[i] ~= nil then
        enemyHealRatio = healingOfKillerByEnemy[i] / healingOfKillerByAllEnemies
      end
      local dHealAssist = (enemyHealRatio > 0.5 and dSolo ~= 1) and 1 or 0
      local healerDetail = Inspect.Unit.Detail(i)
      if not healerDetail then healerDetail = unitDetailsCache[i] end
      if not healerDetail then
        healerDetail = BOTF.getUnknown()
      end
      if dHealAssist == 1 and not pHealAssist then
        pHealAssist = true
        healerUnitId = i
        print("" .. healerDetail.name .. " healed " .. killerDetail.name .. " whilst they killed you")
        BOTF.updateRecord(healerDetail.name, healerDetail.calling, healerDetail.role, healerDetail.level, healerDetail.player, healerDetail.locationName, "Defeat_HealAssists")
      end
    end -- end of for loop

    -- loop through all enemies that have damaged player and work out their contribution
    for i, _ in pairs(damageByEnemyToDeceasedPlayer) do

      local enemyDamageRatio = 0
      if damageByEnemyToDeceasedPlayer[i] ~= nil then
        enemyDamageRatio = damageByEnemyToDeceasedPlayer[i] / damagebyAllEnemiesToDeceasedPlayer
      end
      local enemyDistractRatio = 0
      if damageByDeceasedPlayerToEnemy[i] ~= nil then
        enemyDistractRatio = damageByDeceasedPlayerToEnemy[i] / damageByDeceasedPlayerToAllEnemies
      end

      local dSolo = (enemyDamageRatio == 1 and enemyDistractRatio == 1 and healerUnitId == i) and 1 or 0
      local dDamageAssist = (enemyDamageRatio > 0.5 and dSolo ~= 1) and 1 or 0
      local dDistractAssist = (enemyDistractRatio > 0.5 and dSolo ~= 1) and 1 or 0
      local dKillingBlow = (killerId == i) and 1 or 0

      local damagerDetail = Inspect.Unit.Detail(i)
      if not damagerDetail then damagerDetail = unitDetailsCache[i] end
      if not damagerDetail then
        damagerDetail = BOTF.getUnknown()
      end

      -- check for solo or assist/killingblow
      if dSolo == 1 and not pSolo and killerId ~= deceasedId then
        pSolo = true
        theAlert:setMessage("Solo Kill", killerDetail.name .. " single handedly crushed you", {'SoloKill'})
        BOTF.updateRecord(killerDetail.name, killerDetail.calling, killerDetail.role, killerDetail.level, killerDetail.player, killerDetail.locationName, "Defeat_SoloKills")
      else
        if dKillingBlow == 1 and not pKillingBlow  and killerId ~= deceasedId then
          pKillingBlow = true
          theAlert:setMessage("Killing Blow", killerDetail.name .. " got the killing blow on you", {'KillingBlow'})
          BOTF.updateRecord(killerDetail.name, killerDetail.calling, killerDetail.role, killerDetail.level, killerDetail.player, killerDetail.locationName, "Defeat_KillingBlows")
        elseif dDamageAssist == 1 and not pDamageAssist  and killerId ~= deceasedId then
          pDamageAssist = true
          theAlert:setMessage("Damage Assist", damagerDetail.name .. " assisted in your slaughter", nil)
          BOTF.updateRecord(damagerDetail.name, damagerDetail.calling, damagerDetail.role, damagerDetail.level, damagerDetail.player, damagerDetail.locationName, "Defeat_DamageAssists")
        elseif dDistractAssist == 1 and not pDistractAssist  and killerId ~= deceasedId then
          pDistractAssist = true
          theAlert:setMessage("Damage Assist", damagerDetail.name .. " distracted you whilst you were slaughtered", nil)
          BOTF.updateRecord(damagerDetail.name, damagerDetail.calling, damagerDetail.role, damagerDetail.level, damagerDetail.player, damagerDetail.locationName, "Defeat_DistractAssists")
        end
      end
    end -- end of for loop
  end
end


-- This function returns a template for Unit information when we don't have any information about the unit.
-- return table consisting of unknown unit information.
function BOTF.getUnknown()
  local un = {}
  un.name = "Unknown Enemy"
  un.calling = "unknown"
  un.role = "unknown"
  un.level = "unknown"
  un.player = false
  return un
end

-- Remove anything older than assistWindow seconds
-- @param recents a table of recent damage, assiset, distract or heal events
-- @return table of events that are within the time window, anything older has been removed.
function BOTF.cleanOld(recents)
  local currentTime = Inspect.Time.Real()
  local intime = {}
  for i, v in pairs(recents) do
    if i + assistWindow > currentTime then
      intime[i] = v
    end
  end
  return intime
end

-- Checks if we have a recent entry for the target
-- @param string targetName name of the target
-- @param table recents a table of recent interactions
-- @return the target entry or false if none is found
function BOTF.checkRecent(targetUnitId, recents)
  local cleaned = BOTF.cleanOld(recents)
  for i, v in pairs(cleaned) do
    if targetUnitId == v then
      return v
    end
  end
  return false
end

--Updates a player record
-- @param name the name of the enemy
-- @param _calling the calling of the enemy
-- @param _role the role of the enemy
-- @param _level the level of the enemy
-- @param _theType the type of contribution to add to the records
function BOTF.updateRecord(name, _calling, _role, _level, _player, _location, theType)

  if name == Inspect.Unit.Detail("player").name then
    -- bugger we dont want to reward outselves!!!!
    return
  end


  -- instantiate the tables if they do not exist
  if not BOTF_DB[name] then

    -- hack to deal with skull mobs
    if _level=='??' then
      _level='52'
    end

    BOTF_DB[name] = {}
    BOTF_DB[name]['calling'] = 0
    BOTF_DB[name]['role'] = 0
    BOTF_DB[name]['level'] = 0
    BOTF_DB[name]['player'] = nil
    BOTF_DB[name]['session'] = false
    BOTF_DB[name]['last_seen_time'] = 0
    BOTF_DB[name]['last_seen_location'] = ''
    BOTF_DB[name]['statistics'] = {}
    BOTF_DB[name]['statistics']['Victory_SoloKills'] = 0
    BOTF_DB[name]['statistics']['Victory_KillingBlows'] = 0
    BOTF_DB[name]['statistics']['Victory_DamageAssists'] = 0
    BOTF_DB[name]['statistics']['Victory_DistractAssists'] = 0
    BOTF_DB[name]['statistics']['Victory_HealAssists'] = 0
    BOTF_DB[name]['statistics']['Defeat_SoloKills'] = 0
    BOTF_DB[name]['statistics']['Defeat_KillingBlows'] = 0
    BOTF_DB[name]['statistics']['Defeat_DamageAssists'] = 0
    BOTF_DB[name]['statistics']['Defeat_DistractAssists'] = 0
    BOTF_DB[name]['statistics']['Defeat_HealAssists'] = 0
    BOTF_DB[name]['session_statistics'] = {}
    BOTF_DB[name]['session_statistics']['Victory_SoloKills'] = 0
    BOTF_DB[name]['session_statistics']['Victory_KillingBlows'] = 0
    BOTF_DB[name]['session_statistics']['Victory_DamageAssists'] = 0
    BOTF_DB[name]['session_statistics']['Victory_DistractAssists'] = 0
    BOTF_DB[name]['session_statistics']['Victory_HealAssists'] = 0
    BOTF_DB[name]['session_statistics']['Defeat_SoloKills'] = 0
    BOTF_DB[name]['session_statistics']['Defeat_KillingBlows'] = 0
    BOTF_DB[name]['session_statistics']['Defeat_DamageAssists'] = 0
    BOTF_DB[name]['session_statistics']['Defeat_DistractAssists'] = 0
    BOTF_DB[name]['session_statistics']['Defeat_HealAssists'] = 0
  end

  BOTF_DB[name]['calling'] = _calling
  BOTF_DB[name]['role'] = _role
  BOTF_DB[name]['level'] = _level
  BOTF_DB[name]['session'] = true
  BOTF_DB[name]['player'] = _player
  if not _location then _location=Inspect.Unit.Detail('player').locationName end -- set to player pos if we don't have one
  BOTF_DB[name]['last_seen_location'] = _location
  BOTF_DB[name]['last_seen_time'] = os.time()
  BOTF_DB[name]['statistics'][theType] = BOTF_DB[name]['statistics'][theType] + 1
  BOTF_DB[name]['session_statistics'][theType] = BOTF_DB[name]['session_statistics'][theType] + 1
end

-- @event initialisation after addon initialisation
Command.Event.Attach(Event.Addon.Load.End, 
  function(hEvent,addon) if addon == "BOTF" then addonLoaded()
  end
  end, "Initialise")